from collections import defaultdict

import networkx as nx
import rdflib

from cemento.rdf.turtle import Turtle


class ReadTurtle(Turtle):

    def __init__(self, file_path, include_labels=True):
        super().__init__(file_path=file_path)

        # parse ttl file into graph
        self._rdf_graph = rdflib.Graph()
        self._rdf_graph.parse(self._file_path, format="turtle")

        # read through namespaces and store prefixes
        self._prefixes = {
            str(value): key for key, value in self._rdf_graph.namespaces()
        }

        # get all accepted terms, object properties and classes

        class_terms = set()
        subclass_triples = []
        object_prop_triples = defaultdict(list)
        term_type = defaultdict(list)
        for subj, pred, obj in self._rdf_graph:
            if (
                pred == rdflib.RDFS.subClassOf
                and isinstance(subj, rdflib.URIRef)
                and isinstance(obj, rdflib.URIRef)
            ):
                subclass_triples.append((obj, subj))
                class_terms.update([subj, obj])

            if pred == rdflib.RDF.type:
                term_type[obj].append(subj)

            if pred == rdflib.RDFS.label:
                self._rename_vars[subj].insert(0, obj)

            if pred == rdflib.SKOS.altLabel:
                self._rename_vars[subj].append(obj)

        for obj_prop in term_type[rdflib.OWL.ObjectProperty]:
            for prop_subj, prop_pred, prop_obj in self._rdf_graph.triples(
                (None, obj_prop, None)
            ):
                if isinstance(prop_subj, rdflib.URIRef) and isinstance(
                    prop_obj, rdflib.URIRef
                ):
                    object_prop_triples[prop_pred].append((subj, obj))

        all_classes = {
            term
            for term in self._rdf_graph.subjects(rdflib.RDF.type, rdflib.OWL.Class)
            if isinstance(term, rdflib.URIRef)
        }
        class_terms.update(all_classes - class_terms)

        self._graph = nx.DiGraph()
        self._graph.add_edges_from(subclass_triples, label="rdfs:subClassOf")
        for pred, triples in object_prop_triples.items():
            self._graph.add_edges_from(triples, label=self.get_abbrev_name(pred))
        self._graph.add_nodes_from(class_terms)

        new_term_names = dict()
        for term in all_classes:
            all_names = self.get_all_names(term)
            extra_name = f" ({','.join(all_names)})"
            new_term_names[term] = (
                f"{self.get_abbrev_name(term)}{extra_name if include_labels and all_names else ''}"
            )
        self._graph = nx.relabel_nodes(self._graph, new_term_names)



import rdflib
from cemento.rdf.turtle import Turtle

class WriteTurtle(Turtle):
    
    # TODO: move away from unreliable triples and switch to graph and the fundamental object
    def __init__(self, file_output_path, prefixes, graph):
        super().__init__(file_path=file_output_path, graph=graph, prefixes=prefixes)

        self._rdf_graph = rdflib.Graph()
        namespaces = dict()
        
        # bind namespaces and save to a dictionary
        for url, prefix in prefixes:
            ns = rdflib.Namespace(url)
            self._rdf_graph.bind(prefix, ns)
            namespaces[prefix] = ns
        
        # TODO: parse draw io diagram with termID (name, altnames**)
        # TODO: parse object property relations in a separate page
        # TODO: parse definitions through a separate page, auto-generate/auto-populate if new terms are added


import os
from cemento.draw_io.read_diagram import ReadDiagram

TEST_CASES_FOLDER = "test_cases/in"

if __name__ == "__main__":
    test_files = [file for file in os.scandir(TEST_CASES_FOLDER) if file.name.endswith('.drawio')] 
    for test_file in test_files:
        print(25*"===")
        print(test_file.path)
        read_diagram = ReadDiagram(test_file.path)
        print(read_diagram.get_relationships())
        print(25*'---', '\nerrors:')
        print('\n'.join([e.message for entry in read_diagram.get_errors().values() for e in entry['errors']]))
            


from collections import defaultdict

from rdflib.namespace import split_uri

from cemento.graph import Graph


class Turtle(Graph):

    def __init__(self, file_path, graph=None, prefixes=None):
        super().__init__(
            file_path=file_path,
            rels_df=None,
            graph=graph,
            ref=None,
            do_gen_ids=True,
            infer_rank=False,
        )
        self._file_path = file_path
        self._rdf_graph = None
        self._rename_vars = defaultdict(list)
        self._prefixes = None

    def get_rdf_graph(self):
        return self._rdf_graph

    def get_prefix(self, uri):
        return self._prefixes[uri]

    def _get_rename_vars(self):
        return self._rename_vars

    def get_abbrev_name(self, term):
        uri, abbrev_name = split_uri(term)
        prefix = self.get_prefix(uri)
        abbrev_name = f"{prefix}:{abbrev_name}"
        return abbrev_name

    def get_common_name(self, term):
        try:
            rename_vars = self._get_rename_vars()
            return rename_vars[term][0]
        except (IndexError, KeyError):
            return term

    def get_all_names(self, term):
        return self._rename_vars[term]



from cemento.draw_io.read_area_diagram import ReadAreaDiagram

# NOTE: please run script from root

if __name__ == "__main__":
    exp = ReadAreaDiagram("test_cases/in/read_area_diagram.drawio")
    # retrieves relationships as usual
    print(exp.get_relationships())
    # retrieve area designations (uses ids of elements)
    print(exp.get_area_designations(parse_values=True))


from cemento.tree import Tree

from cemento.draw_io.read_diagram import ReadDiagram
from cemento.draw_io.write_diagram import WriteDiagram

# NOTE: please run script from root

INPUT_PATH = "test_cases/in/ref_diagram.drawio"
OUTPUT_PATH = "test_cases/out/ref_diagram_fixed.drawio"

if __name__ == "__main__":
    read_diagram = ReadDiagram(file_path=INPUT_PATH)
    rel_df = read_diagram.get_relationships()

    print(rel_df.iloc[:,-4:])

    sample_tree = Tree(rels_df=rel_df)
    draw_diagram = WriteDiagram(OUTPUT_PATH)
    sample_tree.draw_tree(draw_diagram)
    draw_diagram.draw()


from cemento.rdf.read_turtle import ReadTurtle
from cemento.tree import Tree
from cemento.draw_io.write_diagram import WriteDiagram

INPUT_PATH = "<INSERT-PATH-HERE>"
OUTPUT_PATH = "<INSERT-PATH-HERE>"

if __name__ == "__main__":
    ex = ReadTurtle(INPUT_PATH)
    tree = Tree(graph=ex.get_graph(), do_gen_ids=True, invert_tree=True)
    diagram = WriteDiagram(OUTPUT_PATH)
    tree.draw_tree(write_diagram=diagram)
    diagram.draw()


from os import path
import pandas as pd
from cemento.tree import Tree
from cemento.draw_io.write_diagram import WriteDiagram

# NOTE: please run script from root

INPUT_FILE_PATH = "test_cases/in/sample_tree.xlsx"
SAVE_FOLDER_PATH = "test_cases/out/sample_tree"

if __name__ == "__main__":

    excel_sheets =  pd.read_excel(INPUT_FILE_PATH, sheet_name=None)

    for sheet_name, df in excel_sheets.items():
        print(sheet_name)
        sample_tree = Tree(rels_df=df)
        diagram_save_path = path.join(SAVE_FOLDER_PATH, f"{sheet_name}.drawio")
        diagram = WriteDiagram(file_path=diagram_save_path)
        sample_tree.draw_tree(write_diagram=diagram)
        diagram.draw()